@@ -17,97 +17,113 @@ VALUE cXMLNode;
 
 /* Memory management:
  *
- * The bindings create a one-to-one mapping between libxml nodes
- * and Ruby nodes.  If a libxml node is wrapped, the mapping is stored in the
- * private_pointers hashtable.
+ * The bindings create a one-to-one mapping between ruby objects and
+ * libxml documents and libxml parent nodes (ie, nodes that do not
+ * have a parent and do not belong to a document). In these cases,
+ * the bindings manage the memory.  They do this by installing a free
+ * function and storing a back pointer to the Ruby object from the xmlnode
+ * using the _private member on libxml structures.  When the Ruby object
+ * goes out of scope, the underlying libxml structure is freed.  Libxml
+ * itself then frees all child node (recursively).
  *
- * When a libxml document or top level node is freed, it will free
- * all its children.  Thus Ruby is responsible for:
+ * For all other nodes (the vast majority), the bindings create temporary
+ * Ruby objects that get freed once they go out of scope. Thus there can be
+ * more than one ruby object pointing to the same xml node.  To mostly hide
+ * this from programmers on the ruby side, the #eql? and #== methods are 
+ * overriden to check if two ruby objects wrap the same xmlnode.  If they do,
+ * then the methods return true.  During the mark phase, each of these temporary
+ * objects marks its owning document, thereby keeping the Ruby document object
+ * alive and thus the xmldoc tree.
  *
- *  * Using the mark function to keep alive any documents Ruby is
- *    referencing via the document or child nodes.
- *  * Using the mark function to keep alive any top level, free
- *    standing nodes Ruby is referencing via the node or its children.
- *
- * In general use, this will cause Ruby nodes to be freed before
- * a libxml document.  When a Ruby node is freed, the hashtable entry is
- * removed.
- *
- * In the sweep phase in Ruby 1.9.*, the document tends to be
- * freed before the nodes.  To support this, the bindings register
- * a callback function with libxml that is called each time a node
- * is freed.  In that case, the data_ptr is set to null, so the bindings
- * can recognize the situation.
+ * In the sweep phase of the garbage collector, or when a program ends, 
+ * there is no order to how Ruby objects are freed. In fact, the ruby document
+ * object is almost always freed before any ruby objects that wrap child nodes.
+ * However, this is ok because those ruby objects do not have a free function
+ * and are no longer in scope (since if they were the document would not be freed).
  */
 
-static void rxml_node_deregisterNode(xmlNodePtr xnode)
-{
-  /* Has the node been wrapped and exposed to Ruby? */
-  VALUE node = rxml_lookup_node(xnode);
-  if (node == Qnil)
-      return;
-
-  /* Node was wrapped. Disassociate the ruby object from the xml node
-     and turn off the free function so Ruby will not call it when the
-     wrapping object is itself freed. Note we still MUST include 
-     the mark function.  Unsetting it breaks the Ruby GC. */
-   RDATA(node)->dfree = NULL;
-   RDATA(node)->data = NULL;
-
-  // Remove the hashtable entry
-  rxml_unregister_node(xnode);
-}
-
 static void rxml_node_free(xmlNodePtr xnode)
 {
-  /* The ruby object wrapping the xml object no longer exists. */
-  rxml_unregister_node(xnode);
-
-  /* Ruby is responsible for freeing this node if it does not
-     have a parent and is not owned by a document.  Note a corner
-     case here - calling node2 = doc.import(node1) will cause node2
-     to not have a parent but to have a document. */
+  /* The ruby object wrapping the xml object no longer exists and this
+     is a standalone node without a document or parent so ruby is 
+     responsible for freeing the underlying node.*/
   if (xnode->doc == NULL && xnode->parent == NULL)
   {
+    // Remove the back linkage from libxml to Ruby
+    xnode->_private = NULL;
     xmlFreeNode(xnode);
   }
 }
 
- void rxml_node_mark(xmlNodePtr xnode)
+void rxml_node_manage(xmlNodePtr xnode, VALUE node)
 {
-   VALUE doc = Qnil;
-   VALUE parent = Qnil;
+  RDATA(node)->dfree = (RUBY_DATA_FUNC)rxml_node_free;
+  xnode->_private = (void*)node;
+}
 
-   /* Either the node has not been created yet in initialize
-     or it has been freed by libxml already in Ruby's 
-     mark phase. */
-  if (xnode == NULL)
-    return;
+void rxml_node_unmanage(xmlNodePtr xnode, VALUE node)
+{
+  RDATA(node)->dfree = NULL;
+  xnode->_private = NULL;
+}
 
-  doc = rxml_lookup_doc(xnode->doc);
-  if (doc != Qnil)
-    rb_gc_mark(doc);
+xmlNodePtr rxml_node_root(xmlNodePtr xnode)
+{
+  xmlNodePtr current = xnode;
 
-  parent = rxml_lookup_node(xnode->parent);
-  if (parent != Qnil)
-     rb_gc_mark(parent);
+  while (current->parent)
+  {
+    current = current->parent;
+  }
+
+  return current;
+}
+
+void rxml_node_mark(xmlNodePtr xnode)
+{
+   if (xnode->doc)
+   {
+     if (xnode->doc->_private)
+	 {
+	   VALUE doc = (VALUE)xnode->doc->_private;
+	   rb_gc_mark(doc);
+	 }
+   }
+   else if (xnode->parent)
+   {
+     xmlNodePtr root = rxml_node_root(xnode);
+     if (root->_private)
+     {
+       VALUE node = (VALUE)root->_private;
+       rb_gc_mark(node);
+     }
+   }
 }
 
 VALUE rxml_node_wrap(xmlNodePtr xnode)
 {
-  VALUE result = rxml_lookup_node(xnode);
+  VALUE result = Qnil;
 
-  if (result == Qnil) {
-    result = Data_Wrap_Struct(cXMLNode, rxml_node_mark, rxml_node_free, xnode);
-    rxml_register_node(xnode, result);
+  // Is this node already wrapped?
+  if (xnode->_private)
+  {
+    result = (VALUE)xnode->_private;
+  }
+  else
+  {
+    result = Data_Wrap_Struct(cXMLNode, rxml_node_mark, NULL, xnode);
+  }
+
+  if (!xnode->doc && !xnode->parent)
+  {
+    rxml_node_manage(xnode, result);
   }
   return result;
 }
 
 static VALUE rxml_node_alloc(VALUE klass)
 {
-  /* Ruby is responsible for freeing this node not libxml but don't set
-     up mark and free yet until we assign the node. */
+  // This node was created from Ruby so we are responsible for freeing it not libxml
   return Data_Wrap_Struct(klass, rxml_node_mark, rxml_node_free, NULL);
 }
 
@@ -270,9 +286,11 @@ static VALUE rxml_node_initialize(int argc, VALUE *argv, VALUE self)
   if (xnode == NULL)
     rxml_raise(&xmlLastError);
 
-  /* Link the Ruby object to the libxml object and vice-versa. */
-  rxml_register_node(xnode, self);
-  DATA_PTR(self) = xnode;
+  // Link the ruby wrapper to the underlying libxml node
+  RDATA(self)->data = xnode;
+
+  // Ruby is in charge of managing this node's memory
+  rxml_node_manage(xnode, self);
 
   if (!NIL_P(content))
     rxml_node_content_set(self, content);
@@ -296,18 +314,18 @@ static VALUE rxml_node_modify_dom(VALUE self, VALUE target,
 
   xmlUnlinkNode(xtarget);
 
-  /* This target node could be freed here. */  
+  // Target is about to have a parent, so stop having ruby manage it.
+  rxml_node_unmanage(xtarget, target);
+
+  // This target node could be freed here and be replaced by a different node 
   xresult = xmlFunc(xnode, xtarget);
 
   if (!xresult)
-    rxml_raise(&xmlLastError);
+	  rxml_raise(&xmlLastError);
 
-  /* Was the target freed? If yes, then wrap the new node */
-  if (xresult != xtarget)
-  {
-    RDATA(target)->data = xresult;
-    rxml_register_node(xresult, target);
-  }
+  /* Assume the target was freed, we need to fix up the ruby object to point to the
+     newly returned node. */
+  RDATA(target)->data = xresult;
 
   return target;
 }
@@ -402,35 +420,6 @@ static VALUE rxml_node_content_set(VALUE self, VALUE content)
   return (Qtrue);
 }
 
-/*
- * call-seq:
- *    node.content_stripped -> "string"
- *
- * Obtain this node's stripped content.
- *
- * *Deprecated*: Stripped content can be obtained via the
- * +content+ method.
- */
-static VALUE rxml_node_content_stripped_get(VALUE self)
-{
-  xmlNodePtr xnode;
-  xmlChar* content;
-  VALUE result = Qnil;
-
-  xnode = rxml_get_xnode(self);
-
-  if (!xnode->content)
-    return result;
-
-  content = xmlNodeGetContent(xnode);
-  if (content)
-  {
-    result = rxml_new_cstr( content, NULL);
-    xmlFree(content);
-  }
-  return (result);
-}
-
 /*
  * call-seq:
  *    node.debug -> true|false
@@ -540,7 +529,7 @@ static VALUE rxml_node_doc(VALUE self)
   if (xdoc == NULL)
     return (Qnil);
 
-  return rxml_lookup_doc(xdoc);
+  return (VALUE)xdoc->_private;
 }
 
 /*
@@ -684,10 +673,10 @@ static VALUE rxml_node_empty_q(VALUE self)
  *    node.eql?(other_node) => (true|false)
  *
  * Test equality between the two nodes. Two nodes are equal
- * if they are the same node or have the same XML representation.*/
+ * if they are the same node.*/
 static VALUE rxml_node_eql_q(VALUE self, VALUE other)
 {
-  if(self == other)
+  if (self == other)
   {
     return Qtrue;
   }
@@ -697,15 +686,9 @@ static VALUE rxml_node_eql_q(VALUE self, VALUE other)
   }
   else
   {
-    VALUE self_xml;
-    VALUE other_xml;
-
-    if (rb_obj_is_kind_of(other, cXMLNode) == Qfalse)
-      rb_raise(rb_eTypeError, "Nodes can only be compared against other nodes");
-
-    self_xml = rxml_node_to_s(0, NULL, self);
-    other_xml = rxml_node_to_s(0, NULL, other);
-    return(rb_funcall(self_xml, rb_intern("=="), 1, other_xml));
+    xmlNodePtr xnode = rxml_get_xnode(self);
+    xmlNodePtr xnode_other = rxml_get_xnode(other);
+    return xnode == xnode_other ? Qtrue : Qfalse;
   }
 }
 
@@ -1125,35 +1108,15 @@ static VALUE rxml_node_property_set(VALUE self, VALUE name, VALUE value)
 
 static VALUE rxml_node_remove_ex(VALUE self)
 {
-  xmlNodePtr xnode, xresult;
-  xnode = rxml_get_xnode(self);
-
-  /* First unlink the node from its parent. */
+  xmlNodePtr xnode = rxml_get_xnode(self);
+ 
+  // Now unlink the node from its parent
   xmlUnlinkNode(xnode);
 
-  /* Now copy the node we want to remove and make the
-     current Ruby object point to it.  We do this because
-     a node has a number of dependencies on its parent
-     document - its name (if using a dictionary), entities,
-     namespaces, etc.  For a node to live on its own, it
-     needs to get its own copies of this information.*/
-  xresult = xmlDocCopyNode(xnode, NULL, 1);
+  // Ruby now manages this node
+  rxml_node_manage(xnode, self);
 
-  /* This ruby node object no longer points at the node.*/
-  rxml_unregister_node(xnode);
-  RDATA(self)->data = NULL;
-
-  /* Now free the original node.  This will call the deregister node
-    callback which would reset the mark and free function except for
-	the fact we already removed it from the private hashtable above */
-  xmlFreeNode(xnode);
-
-  /* Now wrap the new node */
-  RDATA(self)->data = xresult;
-  rxml_register_node(xresult, self);
-
-  /* Now return the removed node so the user can
-     do something with it.*/
+  // Now return the removed node so the user can do something with it
   return self;
 }
 
@@ -1335,12 +1298,6 @@ static VALUE rxml_node_copy(VALUE self, VALUE deep)
 
 void rxml_init_node(void)
 {
-  /* Register callback for main thread */
-  xmlDeregisterNodeDefault(rxml_node_deregisterNode);
-
-  /* Register callback for all other threads */
-  xmlThrDefDeregisterNodeDefault(rxml_node_deregisterNode);
-
   cXMLNode = rb_define_class_under(mXML, "Node", rb_cObject);
 
   rb_define_const(cXMLNode, "SPACE_DEFAULT", INT2NUM(0));
@@ -1419,7 +1376,6 @@ void rxml_init_node(void)
   rb_define_method(cXMLNode, "copy", rxml_node_copy, 1);
   rb_define_method(cXMLNode, "content", rxml_node_content_get, 0);
   rb_define_method(cXMLNode, "content=", rxml_node_content_set, 1);
-  rb_define_method(cXMLNode, "content_stripped", rxml_node_content_stripped_get, 0);
   rb_define_method(cXMLNode, "debug", rxml_node_debug, 0);
   rb_define_method(cXMLNode, "doc", rxml_node_doc, 0);
   rb_define_method(cXMLNode, "empty?", rxml_node_empty_q, 0);
