@@ -50,7 +50,7 @@ static VALUE rxml_parser_context_document(VALUE klass, VALUE document)
     rb_raise(rb_eTypeError, "Must pass an XML::Document object");
 
   Data_Get_Struct(document, xmlDoc, xdoc);
-  xmlDocDumpFormatMemoryEnc(xdoc, &buffer, &length, xdoc->encoding, 0);
+  xmlDocDumpFormatMemoryEnc(xdoc, &buffer, &length, (const char*)xdoc->encoding, 0);
 
   ctxt = xmlCreateDocParserCtxt(buffer);
 
@@ -108,8 +108,7 @@ static VALUE rxml_parser_context_string(VALUE klass, VALUE string)
   if (RSTRING_LEN(string) == 0)
     rb_raise(rb_eArgError, "Must specify a string with one or more characters");
 
-  ctxt = xmlCreateMemoryParserCtxt(StringValuePtr(string),
-                                   RSTRING_LEN(string));
+  ctxt = xmlCreateMemoryParserCtxt(StringValuePtr(string), (int)RSTRING_LEN(string));
   
   if (!ctxt)
     rxml_raise(&xmlLastError);
@@ -188,7 +187,7 @@ static VALUE rxml_parser_context_base_uri_get(VALUE self)
   Data_Get_Struct(self, xmlParserCtxt, ctxt);
 
   if (ctxt->input && ctxt->input->filename)
-    return rxml_new_cstr(ctxt->input->filename, ctxt->encoding);
+    return rxml_new_cstr((const xmlChar*)ctxt->input->filename, ctxt->encoding);
   else
     return Qnil;
 }
@@ -208,8 +207,8 @@ static VALUE rxml_parser_context_base_uri_set(VALUE self, VALUE url)
 
   if (ctxt->input && !ctxt->input->filename)
   {
-    const xmlChar * xurl = StringValuePtr(url);
-    ctxt->input->filename = (char *) xmlStrdup(xurl);
+    const char* xurl = StringValuePtr(url);
+    ctxt->input->filename = (const char*)xmlStrdup((const xmlChar*)xurl);
   }
   return self;
 }
@@ -249,7 +248,7 @@ static VALUE rxml_parser_context_data_directory_get(VALUE self)
   if (ctxt->directory == NULL)
     return (Qnil);
   else
-    return (rxml_new_cstr(ctxt->directory, ctxt->encoding));
+    return (rxml_new_cstr((const xmlChar*)ctxt->directory, ctxt->encoding));
 }
 
 /*
@@ -290,7 +289,7 @@ static VALUE rxml_parser_context_disable_cdata_q(VALUE self)
  *
  * Control whether CDATA nodes will be created in this context.
  */
-static VALUE rxml_parser_context_disable_cdata_set(VALUE self, VALUE bool)
+static VALUE rxml_parser_context_disable_cdata_set(VALUE self, VALUE value)
 {
   xmlParserCtxtPtr ctxt;
   Data_Get_Struct(self, xmlParserCtxt, ctxt);
@@ -299,12 +298,12 @@ static VALUE rxml_parser_context_disable_cdata_set(VALUE self, VALUE bool)
     rb_raise(rb_eRuntimeError, "Sax handler is not yet set");
 
   /* LibXML controls this internally with the default SAX handler. */ 
-  if (bool)
+  if (value)
     ctxt->sax->cdataBlock = NULL;
   else
     ctxt->sax->cdataBlock = xmlDefaultSAXHandler.cdataBlock;
 
-  return bool;
+  return value;
 }
 
 /*
@@ -353,7 +352,7 @@ static VALUE rxml_parser_context_encoding_get(VALUE self)
 {
   xmlParserCtxtPtr ctxt;
   Data_Get_Struct(self, xmlParserCtxt, ctxt);
-  return INT2NUM(xmlParseCharEncoding(ctxt->encoding));
+  return INT2NUM(xmlParseCharEncoding((const char*)ctxt->encoding));
 }
 
 /*
@@ -507,7 +506,7 @@ static VALUE rxml_parser_context_name_node_get(VALUE self)
   if (ctxt->name == NULL)
     return (Qnil);
   else
-    return (rxml_new_cstr((const char*) ctxt->name, ctxt->encoding));
+    return (rxml_new_cstr( ctxt->name, ctxt->encoding));
 }
 
 /*
@@ -534,7 +533,7 @@ static VALUE rxml_parser_context_name_tab_get(VALUE self)
     if (ctxt->nameTab[i] == NULL)
       continue;
     else
-      rb_ary_push(tab_ary, rxml_new_cstr((const char*) ctxt->nameTab[i], ctxt->encoding));
+      rb_ary_push(tab_ary, rxml_new_cstr( ctxt->nameTab[i], ctxt->encoding));
   }
 
   return (tab_ary);
@@ -626,12 +625,11 @@ static VALUE rxml_parser_context_options_get(VALUE self)
  */
 static VALUE rxml_parser_context_options_set(VALUE self, VALUE options)
 {
-  int result;
   xmlParserCtxtPtr ctxt;
   Check_Type(options, T_FIXNUM);
 
   Data_Get_Struct(self, xmlParserCtxt, ctxt);
-  result = xmlCtxtUseOptions(ctxt, NUM2INT(options));
+  xmlCtxtUseOptions(ctxt, NUM2INT(options));
 
   return self;
 }
@@ -661,12 +659,12 @@ static VALUE rxml_parser_context_recovery_q(VALUE self)
  * Control whether recovery mode is enabled in this
  * context.
  */
-static VALUE rxml_parser_context_recovery_set(VALUE self, VALUE bool)
+static VALUE rxml_parser_context_recovery_set(VALUE self, VALUE value)
 {
   xmlParserCtxtPtr ctxt;
   Data_Get_Struct(self, xmlParserCtxt, ctxt);
 
-  if (TYPE(bool) == T_FALSE)
+  if (value == Qfalse)
   {
     ctxt->recovery = 0;
     return (Qfalse);
@@ -703,12 +701,12 @@ static VALUE rxml_parser_context_replace_entities_q(VALUE self)
  * Control whether external entity replacement is enabled in this
  * context.
  */
-static VALUE rxml_parser_context_replace_entities_set(VALUE self, VALUE bool)
+static VALUE rxml_parser_context_replace_entities_set(VALUE self, VALUE value)
 {
   xmlParserCtxtPtr ctxt;
   Data_Get_Struct(self, xmlParserCtxt, ctxt);
 
-  if (TYPE(bool) == T_FALSE)
+  if (value == Qfalse)
   {
     ctxt->replaceEntities = 0;
     return (Qfalse);
@@ -800,7 +798,7 @@ static VALUE rxml_parser_context_subset_name_get(VALUE self)
   if (ctxt->intSubName == NULL)
     return (Qnil);
   else
-    return (rxml_new_cstr((const char*) ctxt->intSubName, ctxt->encoding));
+    return (rxml_new_cstr(ctxt->intSubName, ctxt->encoding));
 }
 
 /*
@@ -819,7 +817,7 @@ static VALUE rxml_parser_context_subset_external_uri_get(VALUE self)
   if (ctxt->extSubURI == NULL)
     return (Qnil);
   else
-    return (rxml_new_cstr((const char*) ctxt->extSubURI, ctxt->encoding));
+    return (rxml_new_cstr( ctxt->extSubURI, ctxt->encoding));
 }
 
 /*
@@ -838,7 +836,7 @@ static VALUE rxml_parser_context_subset_external_system_id_get(VALUE self)
   if (ctxt->extSubSystem == NULL)
     return (Qnil);
   else
-    return (rxml_new_cstr((const char*) ctxt->extSubSystem, ctxt->encoding));
+    return (rxml_new_cstr( ctxt->extSubSystem, ctxt->encoding));
 }
 
 /*
@@ -923,7 +921,7 @@ static VALUE rxml_parser_context_version_get(VALUE self)
   if (ctxt->version == NULL)
     return (Qnil);
   else
-    return (rxml_new_cstr((const char*) ctxt->version, ctxt->encoding));
+    return (rxml_new_cstr( ctxt->version, ctxt->encoding));
 }
 
 /*
