@@ -74,17 +74,17 @@
  *  ['ns0:http://services.somewhere.com', 'ns1:http://domain.somewhere.com'])
 */
 
-
 #include "ruby_libxml.h"
 
 VALUE mXPath;
 
-VALUE
-rxml_xpath_to_value(xmlXPathContextPtr xctxt, xmlXPathObjectPtr xobject) {
+VALUE rxml_xpath_to_value(xmlXPathContextPtr xctxt, xmlXPathObjectPtr xobject)
+{
   VALUE result;
   int type;
 
-  if (xobject == NULL) {
+  if (xobject == NULL)
+  {
     /* xmlLastError is different than xctxt->lastError.  Use
      xmlLastError since it has the message set while xctxt->lastError
      does not. */
@@ -92,7 +92,8 @@ rxml_xpath_to_value(xmlXPathContextPtr xctxt, xmlXPathObjectPtr xobject) {
     rxml_raise(xerror);
   }
 
-  switch (type = xobject->type) {
+  switch (type = xobject->type)
+  {
     case XPATH_NODESET:
       result = rxml_xpath_object_wrap(xctxt->doc, xobject);
       break;
@@ -118,44 +119,49 @@ rxml_xpath_to_value(xmlXPathContextPtr xctxt, xmlXPathObjectPtr xobject) {
   return result;
 }
 
-xmlXPathObjectPtr
-rxml_xpath_from_value(VALUE value) {
+xmlXPathObjectPtr rxml_xpath_from_value(VALUE value)
+{
   xmlXPathObjectPtr result = NULL;
 
-  switch (TYPE(value)) {
-    case T_TRUE:
-    case T_FALSE:
-      result = xmlXPathNewBoolean(RTEST(value));
-      break;
-    case T_FIXNUM:
-    case T_FLOAT:
-      result = xmlXPathNewFloat(NUM2DBL(value));
-      break;
-    case T_STRING:
-      result = xmlXPathWrapString(xmlStrdup((const xmlChar *)StringValuePtr(value)));
-      break;
-    case T_NIL:
-      result = xmlXPathNewNodeSet(NULL);
-      break;
-    case T_ARRAY: {
-      long i, j;
-      result = xmlXPathNewNodeSet(NULL);
+  switch (TYPE(value))
+  {
+  case T_TRUE:
+  case T_FALSE:
+    result = xmlXPathNewBoolean(RTEST(value));
+    break;
+  case T_FIXNUM:
+  case T_FLOAT:
+    result = xmlXPathNewFloat(NUM2DBL(value));
+    break;
+  case T_STRING:
+    result = xmlXPathWrapString(xmlStrdup((const xmlChar *)StringValuePtr(value)));
+    break;
+  case T_NIL:
+    result = xmlXPathNewNodeSet(NULL);
+    break;
+  case T_ARRAY:
+  {
+    long i, j;
+    result = xmlXPathNewNodeSet(NULL);
 
-      for (i = RARRAY_LEN(value); i > 0; i--) {
-        xmlXPathObjectPtr obj = rxml_xpath_from_value(rb_ary_shift(value));
+    for (i = RARRAY_LEN(value); i > 0; i--)
+    {
+      xmlXPathObjectPtr obj = rxml_xpath_from_value(rb_ary_shift(value));
 
-        if ((obj->nodesetval != NULL) && (obj->nodesetval->nodeNr != 0)) {
-          for (j = 0; j < obj->nodesetval->nodeNr; j++) {
-            xmlXPathNodeSetAdd(result->nodesetval, obj->nodesetval->nodeTab[j]);
-          }
+      if ((obj->nodesetval != NULL) && (obj->nodesetval->nodeNr != 0))
+      {
+        for (j = 0; j < obj->nodesetval->nodeNr; j++)
+        {
+          xmlXPathNodeSetAdd(result->nodesetval, obj->nodesetval->nodeTab[j]);
         }
       }
-      break;
     }
-    default:
-      rb_raise(rb_eTypeError,
-        "can't convert object of type %s to XPath object", rb_obj_classname(value)
-      );
+    break;
+  }
+  default:
+    rb_raise(rb_eTypeError,
+      "can't convert object of type %s to XPath object", rb_obj_classname(value)
+    );
   }
 
   return result;
